class Hermes::Api::Dmo::ActivityNotification::NotificationsController < Hermes::Api::Dmo::ActivityNotification::BaseNotificationsController
  protect_from_forgery except: [:open_all]
    rescue_from ::ActivityNotification::NotifiableNotFoundError, with: :render_notifiable_not_found

    # Returns notification index of the target.
    #
    # GET /:target_type/:target_id/notifications
    def index
      super
      render json: {
        count: @notifications.size, 
        notifications: @notifications.as_json(notification_json_options)
      }
    end

    # Opens all notifications of the target.
    #
    # POST /:target_type/:target_id/notifications/open_all
    def open_all
      @notifications = @target.open_all_notifications(params)
      render json: {
        count: @notifications.size,
        notifications: @notifications.as_json(notification_json_options)
      }
    end
  
    # Returns a single notification.
    #
    # GET /:target_type/:target_id/notifications/:id
    def show
      super
      render json: notification_json
    end
  
    # Deletes a notification.
    #
    # DELETE /:target_type/:target_id/notifications/:id
    def destroy
      super
      head 204
    end
  
    # Opens a notification.
    #
    # PUT /:target_type/:target_id/notifications/:id/open
    def open
      with_members = !(params[:with_group_members].to_s.to_boolean(false) || params[:without_grouping].to_s.to_boolean(false))
      @opened_notifications_count = @notification.open!(with_members: with_members)
      unless params[:move].to_s.to_boolean(false)
        render json: {
          count: @opened_notifications_count,
          notification: notification_json
        }
      end
    end

    # Moves to notifiable_path of the notification.
    #
    # GET /:target_type/:target_id/notifications/:id/move
    def move
      super
      render status: 302, location: @notification.notifiable_path, json: {
        location: @notification.notifiable_path,
        count: (@opened_notifications_count || 0),
        notification: notification_json
      }
    end

  protected
  # Custom method to build notification data
  def build_notifications
    @notifications.map do |notification|
      notification['title'] = t("#{notification['key']}.title", title: notification['notifiable_id'])
      notification['body'] = t("#{notification['key']}.body")
      notification
    end
  end

  # Returns options for notification JSON
  # @api protected
  def notification_json_options
    build_notifications
    {}
  end

  # Returns JSON of @notification
  # @api protected
  def notification_json
    @notification.as_json.merge({
      title: t("#{@notification.key}.title", title: @notification.notifiable.id),
      body: t("#{@notification.key}.body")
    })
  end

  # Render associated notifiable record not found error with 500 status
  # @api protected
  # @param [Error] error Error object
  # @return [void]
  def render_notifiable_not_found(error)
    render status: 500, json: error_response(code: 500, message: "Associated record not found", type: error.message)
  end
end